"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const kiss_tnc_1 = __importDefault(require("kiss-tnc"));
const events_1 = require("events");
const Keystore_js_1 = require("./Keystore.js");
const Packet_js_1 = require("./Packet.js");
const utils_js_1 = require("./utils.js");
var Verification;
(function (Verification) {
    Verification[Verification["NotSigned"] = 0] = "NotSigned";
    Verification[Verification["KeyNotFound"] = 1] = "KeyNotFound";
    Verification[Verification["Valid"] = 2] = "Valid";
    Verification[Verification["Invalid"] = 3] = "Invalid";
})(Verification = exports.Verification || (exports.Verification = {}));
class Messenger extends events_1.EventEmitter {
    constructor(config) {
        super();
        this.recentlySent = new Map();
        this.config = config;
        this.ks = new Keystore_js_1.Keystore(this.config.keystoreFile);
        if (this.ks.getKeyPairs(config.callsign).length === 0) {
            this.ks.genKeyPair(config.callsign);
        }
        // device, baud_rate
        this.tnc = this._createTNC(config.kissPort, config.kissBaud);
    }
    openTNC() {
        return new Promise((resolve, reject) => {
            if (this.tnc == null) {
                this.tnc = this._createTNC(this.config.kissPort, this.config.kissBaud);
            }
            this.tnc.open((err) => {
                this.emit('open', err);
                if (err)
                    reject(err);
                else
                    resolve();
            });
        });
    }
    closeTNC() {
        this.tnc.close();
        this.tnc = null;
        this.emit('close');
    }
    send(to, message, sign) {
        return __awaiter(this, void 0, void 0, function* () {
            if (typeof to === 'string')
                to = utils_js_1.callsignSSIDToStation(to);
            const from = { callsign: this.config.callsign, ssid: this.config.ssid };
            let signature = null;
            if (sign) {
                if (this.config.signingKey) {
                    const privates = this.ks.getKeyPairs(this.config.callsign)
                        .filter(key => key.public === this.config.signingKey)
                        .map(key => key.private);
                    if (privates.length > 0) {
                        signature = this.ks.sign(message, privates[0]);
                    }
                    else {
                        throw Error('No signing key was found in the keystore. Make sure your config.signingKey is in your keystore.');
                    }
                }
                else {
                    throw Error(`sign is ${sign} but config.signingKey "${this.config.signingKey}" is not in keystore.`);
                }
            }
            const packet = yield Packet_js_1.Packet.ToAX25Packet(from, to, message, signature);
            if (this.tnc == null)
                throw Error('Error sending message with send(). The Messenger\'s TNC object is null. Are you sure it is connected?');
            return new Promise((resolve, reject) => {
                this.tnc.send_data(packet, (err) => {
                    if (err)
                        reject(err);
                    if (this.config.feedbackDebounce) {
                        this._addToRecentlySent(packet, this.config.feedbackDebounce);
                    }
                    resolve();
                });
            });
        });
    }
    _onAX25DataRecieved(data) {
        return __awaiter(this, void 0, void 0, function* () {
            let packet;
            try {
                packet = yield Packet_js_1.Packet.FromAX25Packet(data.data);
            }
            catch (err) {
                if (err.name == 'InvalidPacket') {
                    // console.log('Received invalid packet, skipping')
                    return;
                }
                else
                    throw err;
            }
            if (this.config.feedbackDebounce && this._wasRecentlySent(data.data)) {
                // console.error('[verbose] received recently sent message, aborting received event.')
                return;
            }
            let verification = Verification.NotSigned;
            if (packet.signature) {
                // if we don't have a public key from that callsign
                if (this.ks.getPublicKeys(packet.from.callsign).length === 0) {
                    verification = Verification.KeyNotFound;
                }
                else {
                    const verified = this.ks.verify(packet.from.callsign, packet.message, packet.signature);
                    if (verified)
                        verification = Verification.Valid;
                    else
                        verification = Verification.Invalid;
                }
            }
            const event = {
                to: { callsign: packet.to.callsign.trim(), ssid: packet.to.ssid },
                from: { callsign: packet.from.callsign.trim(), ssid: packet.from.ssid },
                message: packet.message,
                verification,
                ax25Buffer: data.data
            };
            this.emit('message', event);
        });
    }
    _createTNC(port, baudrate) {
        const tnc = new kiss_tnc_1.default(port, baudrate);
        // process.on('SIGTERM', tnc.close)
        tnc.on('error', (error) => this.emit('tnc-error', error));
        tnc.on('data', (data) => this._onAX25DataRecieved(data));
        return tnc;
    }
    _addToRecentlySent(buffer, expiresIn) {
        // MD5 is weak, but it doesn't matter here because we aren't using it
        // for anything sensitive. Here we like speed ;)
        const fingerprint = utils_js_1.md5(buffer);
        if (!this.recentlySent.has(fingerprint)) {
            this.recentlySent.set(fingerprint, 1);
        }
        else {
            let count = this.recentlySent.get(fingerprint);
            this.recentlySent.set(fingerprint, count + 1);
        }
        utils_js_1.timeout(expiresIn).then(() => {
            const count = this.recentlySent.get(fingerprint);
            if (count == 1)
                this.recentlySent.delete(fingerprint);
            else
                this.recentlySent.set(fingerprint, count - 1);
        });
    }
    _wasRecentlySent(buffer) {
        // MD5 is weak, but it doesn't matter here because we aren't using it
        // for anything sensitive. Here we like speed ;)
        const fingerprint = utils_js_1.md5(buffer);
        return this.recentlySent.has(fingerprint);
    }
}
exports.Messenger = Messenger;
//# sourceMappingURL=Messenger.js.map