"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const AX25 = __importStar(require("ax25")); // https://github.com/echicken/node-ax25/tree/es6rewrite
const compression_js_1 = require("./compression.js");
const utils_js_1 = require("./utils.js");
var HeaderFlags;
(function (HeaderFlags) {
    HeaderFlags[HeaderFlags["Compressed"] = 1] = "Compressed";
    HeaderFlags[HeaderFlags["Signed"] = 2] = "Signed";
})(HeaderFlags = exports.HeaderFlags || (exports.HeaderFlags = {}));
exports.MagicBytes = [0x7a, 0x39];
class Packet {
    constructor() {
        this.header = {
            version: 0x01,
            compressed: null,
            signed: null,
            signatureLength: null
        };
        this.from = null;
        this.to = null;
        this.message = null;
        this.signature = null;
        this.data = null;
    }
    toAX25Packet() {
        return __awaiter(this, void 0, void 0, function* () {
            const packet = new AX25.Packet();
            packet.type = AX25.Masks.control.frame_types.u_frame.subtypes.ui;
            packet.source = this.from;
            packet.destination = this.to;
            packet.payload = yield this.assemble();
            return packet.assemble();
        });
    }
    assemble() {
        return __awaiter(this, void 0, void 0, function* () {
            if (Buffer.isBuffer(this.signature)) {
                this.header.signed = true;
                this.header.signatureLength = this.signature.length;
            }
            // compress signature + message
            let payload = this.header.signed ? this.signature : Buffer.from([]);
            let message = Buffer.from(this.message, 'utf8');
            const compressed = yield compression_js_1.compress(message);
            if (compressed.length < message.length) {
                this.header.compressed = true;
                payload = Buffer.concat([payload, compressed]);
            }
            else {
                this.header.compressed = false;
                payload = Buffer.concat([payload, message]);
            }
            // flags
            let flags = 0x00;
            if (this.header.signed)
                flags = flags | HeaderFlags.Signed;
            if (this.header.compressed)
                flags = flags | HeaderFlags.Compressed;
            // header buffer
            const headerArray = [...exports.MagicBytes, this.header.version, flags];
            if (this.signature !== null) {
                if (this.signature.length > 256)
                    throw Error('signature is larger than 256 bytes');
                headerArray.push(this.signature.length);
            }
            const header = Buffer.from(new Uint8Array(headerArray));
            this.data = Buffer.concat([header, payload]);
            return this.data;
        });
    }
    disassemble(data) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!Buffer.isBuffer(data))
                throw TypeError('Invalid data must be a Buffer');
            if (data.length < 4) {
                throw TypeError('Invalid packet, too few bytes.');
            }
            const magic = data.slice(0, 2);
            if (magic[0] !== exports.MagicBytes[0] || magic[1] !== exports.MagicBytes[1]) {
                throw TypeError(`Invalid magic bytes in packet header. This is not a CV Packet.`);
            }
            const version = data[2];
            if (version !== 1) {
                throw TypeError(`Invalid packet version: ${version}`);
            }
            const flags = data[3];
            this.header.compressed = (flags & HeaderFlags.Compressed) == HeaderFlags.Compressed;
            this.header.signed = (flags & HeaderFlags.Signed) == HeaderFlags.Signed;
            // console.log(`compressed: ${this.header.compressed}`)
            // console.log(`signed: ${this.header.signed}`)
            let payloadIndex = 4;
            if (this.header.signed) {
                this.header.signatureLength = data[4];
                payloadIndex = 5;
            }
            let payload = data.slice(payloadIndex);
            let messageIndex = 0;
            if (this.header.signed) {
                this.signature = payload.slice(0, this.header.signatureLength);
                messageIndex = this.header.signatureLength;
            }
            if (this.header.compressed) {
                this.message = (yield compression_js_1.decompress(payload.slice(messageIndex))).toString('utf8');
            }
            else {
                this.message = payload.slice(messageIndex).toString('utf8');
            }
        });
    }
    static ToAX25Packet(from, to, utf8Text, signature) {
        return __awaiter(this, void 0, void 0, function* () {
            if (typeof from === 'string')
                from = utils_js_1.callsignSSIDToStation(from);
            if (typeof to === 'string')
                to = utils_js_1.callsignSSIDToStation(to);
            const packet = new Packet();
            packet.from = from;
            packet.to = to;
            packet.message = utf8Text;
            if (signature) {
                if (Buffer.isBuffer(signature))
                    packet.signature = signature;
                else
                    throw TypeError(`signature must be a Buffer type, not ${typeof signature}`);
            }
            if (packet.signature) {
                packet.header.signatureLength = packet.signature.length;
            }
            return yield packet.toAX25Packet();
        });
    }
    static FromAX25Packet(ax25Buffer) {
        return __awaiter(this, void 0, void 0, function* () {
            const ax25Packet = new AX25.Packet();
            try {
                ax25Packet.disassemble(ax25Buffer);
            }
            catch (err) {
                throwInvalidPacketError(err);
            }
            const packet = new Packet();
            packet.from = ax25Packet.source;
            packet.to = ax25Packet.destination;
            try {
                yield packet.disassemble(ax25Packet.payload);
            }
            catch (err) {
                throwInvalidPacketError(err);
            }
            return packet;
        });
    }
}
exports.Packet = Packet;
function throwInvalidPacketError(err) {
    const error = TypeError(err.message);
    error.name = 'InvalidPacket';
    throw error;
}
//# sourceMappingURL=Packet.js.map