"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs = __importStar(require("fs"));
const path = __importStar(require("path"));
const os = __importStar(require("os"));
const utils_1 = require("./utils");
const Keystore_1 = require("./Keystore");
exports.defaultChattervoxDir = path.join(os.homedir(), '.chattervox');
exports.defaultConfigPath = path.join(exports.defaultChattervoxDir, 'config.json');
exports.defaultKeystorePath = path.join(exports.defaultChattervoxDir, 'keystore.json');
exports.defaultConfig = {
    version: 3,
    callsign: 'N0CALL',
    ssid: 0,
    keystoreFile: exports.defaultKeystorePath,
    kissPort: 'kiss://localhost:8001',
    kissBaud: 9600,
    feedbackDebounce: 20 * 1000,
};
/** Save a config file as JSON
 * @function save
 * @param  {Config} config
 * @param  {string} configPath?
 * @returns void
 */
function save(config, configPath) {
    validate(config);
    const path = typeof configPath === 'string' ? configPath : exports.defaultConfigPath;
    fs.writeFileSync(path, JSON.stringify(config, null, 4));
}
exports.save = save;
/** Load a config file
 * @function load
 * @param  {string} configPath?
 * @returns Config
 */
function load(configPath) {
    const path = typeof configPath === 'string' ? configPath : exports.defaultConfigPath;
    const conf = JSON.parse(fs.readFileSync(path).toString('utf8'));
    if (migrate(conf))
        save(conf, configPath);
    validate(conf);
    return conf;
}
exports.load = load;
/**
 * Update an older config to a newer one. Transforms the config object in place.
 * @function migrate
 * @param {Config} config
 * @returns boolean True if the config was changed
 */
function migrate(config) {
    let changed = false;
    // this change was made in config v3
    if (typeof config.feedbackDebounce === 'undefined') {
        config.feedbackDebounce = exports.defaultConfig.feedbackDebounce;
        changed = true;
    }
    return changed;
}
exports.migrate = migrate;
/** Check if the config file (or any file) exists
 * @function exists
 * @param  {string} configPath?
 * @returns boolean
 */
function exists(configPath) {
    const path = typeof configPath === 'string' ? configPath : exports.defaultConfigPath;
    return fs.existsSync(path);
}
exports.exists = exists;
/**
 * @function validate
 * @param config
 * @throws TypeError
 */
function validate(config) {
    if (typeof config !== 'object') {
        throw TypeError('config is not an object');
    }
    else if (typeof config.version !== 'number') {
        throw TypeError('version must be a number type');
    }
    else if (typeof config.callsign !== 'string') {
        throw TypeError('callsign must be a string type');
    }
    else if (utils_1.isCallsignSSID(config.callsign)) {
        throw TypeError('callsign must be a valid callsign excluding an SSID');
    }
    else if (!utils_1.isCallsign(config.callsign)) {
        throw TypeError('callsign must be a valid callsign');
    }
    else if (!utils_1.isSSID(config.ssid)) {
        throw TypeError('ssid must be a number between 0 and 15');
    }
    else if (typeof config.kissPort !== 'string') {
        throw TypeError('kissPort must be a string type');
    }
    else if (typeof config.kissBaud !== 'number') {
        throw TypeError('kissBaud must be a number type');
    }
    else if (typeof config.keystoreFile !== 'string') {
        throw TypeError('keystoreFile must be a string type');
    }
    else if (typeof config.signingKey !== 'undefined'
        && config.signingKey !== null
        && typeof config.signingKey !== 'string') {
        throw TypeError('signingKey must be a string or null if it is defined');
    }
    else if (config.feedbackDebounce !== null
        && typeof config.feedbackDebounce !== 'number') {
        throw TypeError('feedbackDebounce must be a number or null');
    }
}
exports.validate = validate;
/**
 * Create new chattervox directory, config file, and keystore ONLY if they do
 * not already exist.
 * @function init
 */
function init() {
    if (!fs.existsSync(exports.defaultChattervoxDir)) {
        fs.mkdirSync(exports.defaultChattervoxDir);
    }
    if (!exists(exports.defaultConfigPath)) {
        save(exports.defaultConfig);
    }
    if (!exists(exports.defaultKeystorePath)) {
        // simply creating a new keystore object with save the store
        new Keystore_1.Keystore(exports.defaultKeystorePath);
    }
}
exports.init = init;
//# sourceMappingURL=config.js.map